/*
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2004-2005 Antoine Beyeler <antoine.beyeler@epfl.ch>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Mav.h"

/*! \file Mav.cpp
	\brief Implementation of the Mav robot
*/

namespace Enki
{
	using namespace An;

	Mav::Mav(unsigned capabilities) :
		comm(100.0, this)
	{
		mass = 1;
		height = 0.01;
		r = 0.5;
		collisionAngularFrictionFactor = 0.7;
		viscousFrictionTau = 0.5;
		viscousMomentFrictionTau = 0;
		if (capabilities & CAPABILITIY_BASIC_SENSORS)
		{
			addLocalInteraction(&comm);
		}
		speed = roll = currentRoll=0;
		currentRollSet=false;
	}
	
	Mav::~Mav()
	{
	}
	void Mav::step(double dt) 
	{	
		// +/- 5% motor noise
		double realSpeed = forwardSpeed* (0.9 + random.getRange(0.1));
		double realAngularSpeed;
		double rand = An::uniformRand();
		double val = (rand*M_PI/18.0)-(M_PI/36.0);
		if(!currentRollSet)
		{
			currentRollSet=true;
			currentRoll=roll;
		}
		if(roll>currentRoll)
		{
			currentRoll+=M_PI/36.0;
		}
		else if(roll<currentRoll)
		{
			currentRoll-=M_PI/36.0;
		}
		currentRoll+=val;
		if (realSpeed != 0.0)
			realAngularSpeed = (tan(currentRoll)*mass*9.81)/realSpeed;
		else
			realAngularSpeed =0.0;
		
		// speeds, according to Prof. Siegwart class material
		double forwardSpeed = realSpeed;
		angSpeed = realAngularSpeed;
		speed = Vector(
			forwardSpeed * cos(angle + angSpeed * dt),
			forwardSpeed * sin(angle + angSpeed * dt));
		
		// PhysicalObject::step will actually move, and in next loop
		// care will be taken regarding collision. So we have to compute
		// the difference here.
		// handle physic	
		PhysicalObject::step(dt);
	}
}

