/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2004-2005 Antoine Beyeler <antoine.beyeler@epfl.ch>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MAV_H
#define __MAV_H

#include <enki/PhysicalEngine.h>
#include <enki/interactions/IRSensor.h>
#include <enki/interactions/CircularCam.h>
#include <enki/robots/mav/MavCommunication.h>

/*!	\file Mav.h
	\brief Header of the Mav robot
*/
	
namespace Enki
{
	//! A simple model of the Mav robot. 
	/*! \ingroup robot */
	class Mav : public Robot
	{
	public:
		MavCommunication comm;
		//! The planes roll and speed m/s
		double roll, forwardSpeed, currentRoll;
		bool currentRollSet;
		enum Capabilities
		{
			//! The bot's capabilities. You can simply select a predefined set of sensors. These correspond to the different extension modules that exist for the Mav.
			CAPABILITIY_NONE = 0,
			//! Basic_Sensors: Just the n communication signals of the base module
			CAPABILITIY_BASIC_SENSORS = 0x1,
		};
		virtual ~Mav();
		//! Create a Mav with certain modules aka capabilities (basic)
		Mav(unsigned capabilities = CAPABILITIY_BASIC_SENSORS);
		//! Call Mav::controlStep and do all the calculations
		void step(double dt);
	};
}

#endif

