/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2004-2005 Markus Waibel <markus.waibel@epfl.ch>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "MavCommunication.h"
#include "Mav.h"

#include <iostream>
#include <sstream>
#include <limits>

#include <assert.h>

/*! \file MavCommunication.cpp
	\brief Implementation of the mav's communication interaction 
*/

namespace Enki
{
	using namespace An;
	
	MavCommunication::MavCommunication(double r, Robot *owner)
	{
		this->r = r;
		this->owner =dynamic_cast<Mav *>(owner);
		enabled = false;
		std::valarray<int> val(0);
		MavMessage *com = new MavMessage(-1,-1,-1.0, false,"",-1,val);
		receivedValue.resize(0);
		transmitValue = com;
		wasReceived = false;
	}
 
	void MavCommunication::init()
	{
		wasReceived = false;
		receivedValue.resize(0);
	}
	
	
	void MavCommunication::objectStep(double dt, PhysicalObject *po, World *w)
	{
		Mav *other = dynamic_cast<Mav *>(po);
		if ((other) && (other->comm.enabled))
		{
			//test if message already received
			bool redundantMessage=false;
			for(size_t i = 0; i<receivedValue.size();i++)
			{
				if(receivedValue[i]->getId()==other->comm.transmitValue->getId()){
					redundantMessage=true;
					break;
				}
			}
			if(!redundantMessage){
				double dist = (owner->pos - other->pos).norm();
				if (dist < r*(0.9 + random.getRange(0.1)))
				{
					// We have communication
					std::valarray<MavMessage *> tempmessages = receivedValue;
					receivedValue.resize(tempmessages.size()+1);
					for(size_t i = 0; i<tempmessages.size();i++)
						receivedValue[i] = tempmessages[i];
					receivedValue[receivedValue.size()-1] = other->comm.transmitValue;
					wasReceived = true;
	
					tempmessages.resize(other->comm.receivedValue.size());
					tempmessages = other->comm.receivedValue;
					other->comm.receivedValue.resize(tempmessages.size()+1);
					for(size_t i = 0; i<tempmessages.size();i++)
						other->comm.receivedValue[i] = tempmessages[i];
					other->comm.receivedValue[other->comm.receivedValue.size()-1] = transmitValue;
					other->comm.wasReceived=true;
				}
				else
				{
					wasReceived = false;
					other->comm.wasReceived=false;
				}
			}
		}
	}

	void MavCommunication::setTransmitValue(MavMessage *val)
	{
		transmitValue = val;
	}

	MavMessage * MavCommunication::getTransmitValue(void)
	{
		return transmitValue;
	}
	void MavCommunication::setTransmit(bool enabled)
	{
		this->enabled = enabled;
	}
	
	bool MavCommunication::wasCommunication(void)
	{
		return wasReceived;
	}
		
	bool MavCommunication::wasCommunicationSent(void)
	{
		return enabled;
	}
	
	std::valarray<MavMessage *> MavCommunication::getSensorValue(void)
	{
		return receivedValue;
	}
}
