/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2004-2005 Markus Waibel <markus.waibel@epfl.ch>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MAV_COMMUNICATION_H
#define __MAV_COMMUNICATION_H

#include <enki/PhysicalEngine.h>
#include <enki/Interaction.h>
#include <enki/robots/mav/MavMessage.h>
/*!
	\file MavCommunication.h
	\brief Header of the mav communication interaction
*/

namespace Enki
{
	//! Communication module for the Mav
	/*! In an mav, the communication is done via messages sent through a wireless RF link.
		\ingroup interaction
	*/
class MavCommunication : public LocalInteraction
	{
	protected:
		//! Is communication active
		bool enabled;
		//! Value to transmit through communication
		MavMessage *transmitValue;
		
		//! Was communication active last step
		bool wasReceived;
		//! Value received through communication
		std::valarray<MavMessage *> receivedValue;
	public:
		//! Constructor; r is the mav's maximum communication range
		MavCommunication(double r, Robot *owner);
		//! Init routine run before the interaction
		void init();
		//! The actual interaction
		void objectStep(double dt, PhysicalObject *po, World *w);
		//! The value to transmit
		void setTransmitValue(MavMessage *val);
		//! The value to transmit
		MavMessage *getTransmitValue(void);
		//! Turn transmission on or off
		void setTransmit(bool enabled);
		//! Returns true if a communication signal was received
		bool wasCommunication(void);
		//! Returns true if a communication signal was sent
		bool wasCommunicationSent(void);
		//! Returns the received messages
		std::valarray<MavMessage *> getSensorValue(void);
	};
}

#endif

