/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MAV_MESSAGE_H
#define __MAV_MESSAGE_H
#include <valarray>
/*!
	\file MavMessage.h
	\brief Header of the mav message entity 
*/
namespace Enki
{
	//! A simple model of message sent from one mav to another.
	class MavMessage
	{
	protected:
		//! The id of the sender mav
		int id;
		//! The number of mavs that can communicate with the sender mav
		int density;
		//! The global track of the sender mav
		double orientation;
		//! True if the sender mav is orbitting, false otherwise
		bool orbitting;
		//! An arbitrary message
		char *msg;
		//! The mav that is currently being saved by the sender mav
		int savedMav;
		//! The table containing the ids of all neighbors
		std::valarray<int> neighborTable;

	public:
		//! Create a message containing all the information to be sent.
		MavMessage(int id,int density, double orientation, bool orbitting, char *msg, int savedMav, std::valarray<int> neighborTable);
		//! The id of the sender mav
		void setId(int id);
		//! The number of mavs that can communicate with the sender mav
		void setDensity(int density);
		//! The global track of the sender mav
		void setOrientation(double orientation);
		//! An arbitrary message relevant to the different algorithms used
		void setMsg(char *msg);
		//! The mav that is currently being saved by the  sender mav
		void setSavedMav(int id);
		//! True if the sender mav is orbitting, false otherwise
		void setOrbitting(bool orbitting);
		//! Table containing the ids of all neighbors
		void setNeighborTable(std::valarray<int> neighborTable);
		
		//! Returns the id of the sender mav
		int getId();
		//! Returns the number of mavs that can communicate with the sender mav
		int getDensity();
		//! Returns the global track of the sender mav
		double getOrientation();
		//! True if the sender mav is orbitting, false otherwise
		bool isOrbitting();
		//! Returns the message
		char *getMsg();
		//! Returns the mav that is currently being saved by the sender mav
		int getSavedMav();
		//! Returns the table containing the ids of all neighbors
		std::valarray<int> getNeighborTable();
	};
}

#endif

